@@ -23,12 +23,18 @@
 #define NETWORK_MANAGER_POLICY_H
 
 #include "NetworkManager.h"
-#include "NetworkManagerDevice.h"
+#include "nm-device.h"
+#include "NetworkManagerDbus.h"
+#include "nm-activation-request.h"
 
-gboolean		nm_state_modification_monitor			(gpointer user_data);
+void			nm_policy_schedule_device_change_check	(NMData *data);
 
-void			nm_policy_update_allowed_access_points	(NMData *data);
+void			nm_policy_schedule_device_activation	(NMActRequest *req);
 
-gpointer		nm_policy_allowed_ap_refresh_worker	(gpointer user_data);
+void			nm_policy_schedule_allowed_ap_list_update (NMData *app_data);
+void			nm_policy_schedule_device_ap_lists_update_from_allowed	(NMData *app_data);
+
+void			nm_policy_schedule_activation_finish	(NMActRequest *req);
+void			nm_policy_schedule_activation_failed	(NMActRequest *req);
 
 #endif
