@@ -170,6 +170,8 @@ static DBusMessage *nm_dbus_nm_activate_dialup (DBusConnection *connection, DBus
 	if (!nm_system_activate_dialup (nm_data->dialup_list, dialup))
 		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "ActivationFailed",
 									   "Failed to activate the dialup device.");
+	else
+		nm_data->modem_active = TRUE;
 	nm_unlock_mutex (nm_data->dialup_list_mutex, __FUNCTION__);
 
 out:
@@ -203,6 +205,8 @@ static DBusMessage *nm_dbus_nm_deactivate_dialup (DBusConnection *connection, DB
 	if (!nm_system_deactivate_dialup (nm_data->dialup_list, dialup))
 		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "DeactivationFailed",
 									   "Failed to deactivate the dialup device.");
+	else
+		nm_data->modem_active = FALSE;
 	nm_unlock_mutex (nm_data->dialup_list_mutex, __FUNCTION__);
 
 out:
@@ -224,8 +228,8 @@ static DBusMessage *nm_dbus_nm_set_active_device (DBusConnection *connection, DB
 	NMDevice *		dev = NULL;
 	DBusMessage *		reply = NULL;
 	char *			dev_path;
-	char *			unescaped_dev_path = NULL;
 	NMAccessPoint *	ap = NULL;
+	NMActRequest * req;
 	DBusMessageIter	iter;
 
 	g_return_val_if_fail (connection != NULL, NULL);
@@ -243,9 +247,7 @@ static DBusMessage *nm_dbus_nm_set_active_device (DBusConnection *connection, DB
 	}
 
 	dbus_message_iter_get_basic (&iter, &dev_path);
-	unescaped_dev_path = nm_dbus_unescape_object_path (dev_path);
-	dev = nm_dbus_get_device_from_object_path (data->data, unescaped_dev_path);
-	g_free (unescaped_dev_path);
+	dev = nm_dbus_get_device_from_escaped_object_path (data->data, dev_path);
 
 	/* Ensure the device exists in our list and is supported */
 	if (!dev || !(nm_device_get_capabilities (dev) & NM_DEVICE_CAP_NM_SUPPORTED))
@@ -303,7 +305,14 @@ static DBusMessage *nm_dbus_nm_set_active_device (DBusConnection *connection, DB
 
 	nm_device_deactivate (dev);
 	nm_schedule_state_change_signal_broadcast (data->data);
-	nm_policy_schedule_device_activation (nm_act_request_new (data->data, dev, ap, TRUE));
+	req = nm_act_request_new (data->data, dev, ap, TRUE);
+	nm_policy_schedule_device_activation (req);
+	nm_act_request_unref (req);
+
+	/* empty success message */
+	reply = dbus_message_new_method_return (message);
+	if (!reply)
+		nm_warning ("Could not allocate dbus message.");
 
 out:
 	if (!reply)
@@ -328,10 +337,10 @@ static DBusMessage *nm_dbus_nm_create_wireless_network (DBusConnection *connecti
 	NMDevice *		dev = NULL;
 	DBusMessage *		reply = NULL;
 	char *			dev_path = NULL;
-	char *			unescaped_dev_path = NULL;
 	NMAccessPoint *	new_ap = NULL;
 	NMAPSecurity * 	security = NULL;
 	char *			essid = NULL;
+	NMActRequest * req;
 	DBusMessageIter	iter;
 
 	g_return_val_if_fail (connection != NULL, NULL);
@@ -348,9 +357,7 @@ static DBusMessage *nm_dbus_nm_create_wireless_network (DBusConnection *connecti
 	}
 
 	dbus_message_iter_get_basic (&iter, &dev_path);
-	unescaped_dev_path = nm_dbus_unescape_object_path (dev_path);
-	dev = nm_dbus_get_device_from_object_path (data->data, unescaped_dev_path);
-	g_free (unescaped_dev_path);
+	dev = nm_dbus_get_device_from_escaped_object_path (data->data, dev_path);
 
 	/* Ensure the device exists in our list and is supported */
 	if (!dev || !(nm_device_get_capabilities (dev) & NM_DEVICE_CAP_NM_SUPPORTED))
@@ -391,7 +398,9 @@ static DBusMessage *nm_dbus_nm_create_wireless_network (DBusConnection *connecti
 	g_object_unref (G_OBJECT (security));
 	nm_ap_set_user_created (new_ap, TRUE);
 
-	nm_policy_schedule_device_activation (nm_act_request_new (data->data, dev, new_ap, TRUE));
+	req = nm_act_request_new (data->data, dev, new_ap, TRUE);
+	nm_policy_schedule_device_activation (req);
+	nm_act_request_unref (req);
 
 out:
 	return reply;
@@ -448,9 +457,7 @@ static DBusMessage *nm_dbus_nm_remove_test_device (DBusConnection *connection, D
 	{
 		NMDevice	*dev;
 
-		dev_path = nm_dbus_unescape_object_path (dev_path);
-
-		if ((dev = nm_dbus_get_device_from_object_path (data->data, dev_path)))
+		if ((dev = nm_dbus_get_device_from_escaped_object_path (data->data, dev_path)))
 		{
 			if (nm_device_is_test_device (dev))
 				nm_remove_device (data->data, dev);
@@ -463,8 +470,6 @@ static DBusMessage *nm_dbus_nm_remove_test_device (DBusConnection *connection, D
 			reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "DeviceNotFound",
 							"The requested network device does not exist.");
 		}
-
-		g_free (dev_path);
 	}
 	else
 	{
@@ -538,8 +543,29 @@ static DBusMessage *nm_dbus_nm_sleep (DBusConnection *connection, DBusMessage *m
 	if (app_data->asleep == FALSE)
 	{
 		GSList *elt;
+		DBusMessageIter iter;
 
-		nm_info ("Going to sleep.");
+		dbus_message_iter_init (message, &iter);
+
+		switch (dbus_message_iter_get_arg_type (&iter)) {
+		case DBUS_TYPE_INVALID:
+			/* The boolean argument to differentiate between sleep and disabling networking
+			   is optional and defaults to sleep */
+			app_data->disconnected = FALSE;
+			break;
+		case DBUS_TYPE_BOOLEAN:
+			dbus_message_iter_get_basic (&iter, &app_data->disconnected);
+			break;
+		default:
+			return nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "InvalidArguments",
+												 "NetworkManager::sleep called with invalid arguments.");
+			break;
+		}
+
+		if (app_data->disconnected)
+			nm_info ("Disconnected.");
+		else
+			nm_info ("Going to sleep.");
 
 		app_data->asleep = TRUE;
 		/* Not using nm_schedule_state_change_signal_broadcast() here
@@ -560,6 +586,7 @@ static DBusMessage *nm_dbus_nm_sleep (DBusConnection *connection, DBusMessage *m
 
 		nm_lock_mutex (app_data->dialup_list_mutex, __FUNCTION__);
 		nm_system_deactivate_all_dialup (app_data->dialup_list);
+		app_data->modem_active = FALSE;
 		nm_unlock_mutex (app_data->dialup_list_mutex, __FUNCTION__);
 	}
 
@@ -569,14 +596,37 @@ static DBusMessage *nm_dbus_nm_sleep (DBusConnection *connection, DBusMessage *m
 static DBusMessage *nm_dbus_nm_wake (DBusConnection *connection, DBusMessage *message, NMDbusCBData *data)
 {
 	NMData	*app_data;
+	DBusMessageIter iter;
+	gboolean enable_networking = FALSE;
 
 	g_return_val_if_fail (data && data->data && connection && message, NULL);
 
+	dbus_message_iter_init (message, &iter);
+
+	switch (dbus_message_iter_get_arg_type (&iter)) {
+	case DBUS_TYPE_INVALID:
+		/* The boolean argument to differentiate between wake up from sleep and
+		   enabling networking is optional and defaults to wake up. */
+		break;
+	case DBUS_TYPE_BOOLEAN:
+		dbus_message_iter_get_basic (&iter, &enable_networking);
+		break;
+	default:
+		return nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "InvalidArguments",
+											 "NetworkManager::wake called with invalid arguments.");
+		break;
+	}
+
 	app_data = data->data;
-	if (app_data->asleep == TRUE)
-	{
-		nm_info  ("Waking up from sleep.");
-		app_data->asleep = FALSE;
+	/* Restore networking only if we're not disconnected or
+	   enable_networking argument is passed. */
+	if (app_data->asleep && (!app_data->disconnected || enable_networking)) {
+		if (enable_networking)
+			nm_info ("Enabling networking.");
+		else
+			nm_info ("Waking up from sleep.");
+
+		app_data->asleep = app_data->disconnected = FALSE;
 
 		/* Remove all devices from the device list */
 		nm_lock_mutex (app_data->dev_list_mutex, __FUNCTION__);
